import os
from blueness import module
import abcli
from abcli import file
from abcli.file.functions import build_from_template
from abcli.plugins import markdown
from blue_geo import NAME, VERSION, ICON
from blue_geo.logger import logger

NAME = module.name(__file__, NAME)

features = {
    "QGIS": {
        "description": "an AI terraform for [QGIS](https://www.qgis.org/).",
        "icon": ICON,
        "thumbnail": "https://raw.githubusercontent.com/kamangir/assets/main/blue-geo/QGIS.jpg",
        "url": "https://github.com/kamangir/blue-geo/blob/main/blue_geo/.abcli/QGIS/README.md",
    },
    "datacube": {
        "description": "a datacube for geospatial AI",
        "icon": "🧊",
        "thumbnail": "https://github.com/kamangir/assets/blob/main/blue-geo/datacube.png?raw=true",
        "url": "https://github.com/kamangir/blue-geo/tree/main/blue_geo/.abcli/datacube",
    },
    "firms-area": {
        "description": "Fire Information for Resource Management System ([FIRMS](https://firms.modaps.eosdis.nasa.gov)) datacubes.",
        "icon": ICON,
        "thumbnail": "https://raw.githubusercontent.com/kamangir/assets/main/blue-geo/datacube-firms_area.jpg",
        "url": "https://github.com/kamangir/blue-geo/blob/main/blue_geo/.abcli/datacube/firms_area/README.md",
    },
    "ukraine-timemap": {
        "description": "`ingest` for the [Bellingcat](https://www.bellingcat.com/) [Civilian Harm in Ukraine TimeMap](https://github.com/bellingcat/ukraine-timemap) dataset, available through [this UI](https://ukraine.bellingcat.com/) and [this API](https://bellingcat-embeds.ams3.cdn.digitaloceanspaces.com/production/ukr/timemap/api.json).",
        "icon": "🇺🇦",
        "thumbnail": "https://github.com/kamangir/assets/blob/main/nbs/ukraine-timemap/QGIS.png?raw=true",
        "url": "https://github.com/kamangir/blue-geo/blob/main/blue_geo/.abcli/ukraine-timemap/README.md",
    },
    "vancouver-watching": {
        "description": "🌈 Vancouver watching with AI, last build: [🔗](https://kamangir-public.s3.ca-central-1.amazonaws.com/test_vancouver_watching_ingest/animation.gif).",
        "icon": "🌈",
        "thumbnail": "https://kamangir-public.s3.ca-central-1.amazonaws.com/test_vancouver_watching_ingest/animation.gif?raw=true",
        "url": "https://github.com/kamangir/Vancouver-Watching",
    },
    "template": {
        "description": "",
        "icon": "",
        "thumbnail": "",
        "url": "",
    },
}


def build(filename: str = ""):
    if not filename:
        filename = os.path.join(file.path(__file__), "../README.md")

    logger.info(f"{NAME}.build: {filename}")

    items = [
        "{}[`{}`]({}) [![image]({})]({}) {}".format(
            details["icon"],
            feature,
            details["url"],
            details["thumbnail"],
            details["url"],
            details["description"],
        )
        for feature, details in features.items()
        if feature != "template"
    ]

    table = markdown.generate_table(items, cols=2)

    signature = [
        "---",
        "built by [`{}`]({}), based on [`{}-{}`]({}).".format(
            abcli.fullname(),
            "https://github.com/kamangir/awesome-bash-cli",
            NAME,
            VERSION,
            "https://github.com/kamangir/blue-geo",
        ),
    ]

    return file.build_from_template(
        os.path.join(file.path(__file__), "./assets/README.md"),
        {
            "--table--": table,
            "--signature": signature,
        },
        filename,
    )



import argparse
from blueness import module
from blue_geo import NAME, VERSION
from blue_geo.catalog.classes import list_of_catalogs, get_collections
from blue_geo.logger import logger
from blueness.argparse.generic import sys_exit

NAME = module.name(__file__, NAME)

list_of_tasks = "get,list"


parser = argparse.ArgumentParser(NAME, description=f"{NAME}-{VERSION}")
parser.add_argument(
    "task",
    type=str,
    help=list_of_tasks,
)
parser.add_argument(
    "--what",
    default="",
    type=str,
    help="list_of_collections",
)
parser.add_argument(
    "--delim",
    type=str,
    default=",",
)
parser.add_argument(
    "--count",
    type=int,
    default=-1,
    help="-1: all",
)
parser.add_argument(
    "--log",
    default=1,
    type=int,
    help="0|1",
)
parser.add_argument(
    "--catalog",
    default=list_of_catalogs[0],
    type=str,
    help="|".join(list_of_catalogs),
)
args = parser.parse_args()

delim = " " if args.delim == "space" else args.delim

success = args.task in list_of_tasks
item_name = "item"
if args.task == "get":
    output = []

    if args.what == "list_of_collections":
        item_name = "collection"
        output = [
            datacube_class.name for datacube_class in get_collections(args.catalog)
        ]

    if args.count != -1:
        output = output[: args.count]
elif args.task == "list":
    item_name = "catalog"
    output = list_of_catalogs
else:
    success = None

if success:
    if args.log:
        logger.info(
            "{:,} {}(s): {}".format(
                len(output),
                item_name,
                delim.join(output),
            )
        )
    else:
        print(delim.join(output))

sys_exit(logger, NAME, args.task, success)



from argparse import ArgumentParser
from abcli.logger import ABCUL
from typing import Dict


def add_default_arguments(
    default_args: Dict[str, Dict],
    parser: ArgumentParser,
):
    for arg, values in default_args.items():
        parser.add_argument(
            f"--{arg}",
            type=values.get("type", str),
            default=values["default"],
            help=values["help"],
        )


def as_list_of_args(default_args: Dict[str, Dict]) -> str:
    return ABCUL.join(
        sorted(
            [
                "[--{} {}]".format(
                    arg,
                    values["help"],
                )
                for arg, values in default_args.items()
            ]
        )
    )



import argparse
from blueness import module
from blue_geo import NAME, VERSION
from blue_geo.ukraine_timemap.functions import ingest
from blue_geo.logger import logger
from blueness.argparse.generic import sys_exit

NAME = module.name(__file__, NAME)


parser = argparse.ArgumentParser(NAME, description=f"{NAME}-{VERSION}")
parser.add_argument(
    "task",
    type=str,
    help="ingest",
)
parser.add_argument(
    "--object_name",
    type=str,
)
args = parser.parse_args()

success = False
if args.task == "ingest":
    success, _ = ingest(object_name=args.object_name)
else:
    success = None

sys_exit(logger, NAME, args.task, success)



from abcli import file, fullname, string
from datetime import datetime
from collections import Counter
from typing import Tuple
from abcli.modules import objects
import geopandas as gpd
from geojson import Point
from blueness import module
from blue_geo import NAME, VERSION
from blue_geo.logger import logger
import matplotlib.pyplot as plt
from typing import Dict
from typing import Any

API_URL = "https://bellingcat-embeds.ams3.cdn.digitaloceanspaces.com/production/ukr/timemap/api.json"

DESCRIPTION = "Civilian Harm in Ukraine TimeMap"


NAME = module.name(__file__, NAME)


def ingest(
    object_name: str,
    do_save: bool = True,
    do_visualize: bool = True,
    log: bool = True,
) -> Tuple[bool, gpd.GeoDataFrame]:
    logger.info(f"{NAME}.ingest -> {object_name}")
    filename = objects.path_of(
        "ukraine_timemap.json",
        object_name,
        create=True,
    )

    gdf = gpd.GeoDataFrame()
    metadata: Dict[str, Any] = {
        "description": DESCRIPTION,
        "created_by": f"{NAME}-{VERSION}.{fullname()}",
        "creation_date": string.pretty_date(),
    }

    success = file.download(API_URL, filename)
    if not success:
        return success, gdf

    success, list_of_events = file.load_json(filename)
    if not success:
        return success, gdf
    logger.info("{:,} event(s) ingested from the api.".format(len(list_of_events)))
    metadata["api_count"] = len(list_of_events)

    records = []
    failure_count = 0
    for event in list_of_events:
        try:
            point = Point(
                (
                    float(event["longitude"]),
                    float(event["latitude"]),
                )
            )
            record = {
                "geometry": point,
                "sources": ", ".join(event["sources"]),
                "id": event["id"],
                "description": event["description"],
                "date": event["date"],
                "date_obj": datetime.strptime(event["date"], "%m/%d/%Y").date(),
                "location": event["location"],
                "graphic": event["graphic"],
                "associations": ", ".join(event["associations"]),
                "time": event["time"],
            }
        except Exception as e:
            logger.error(f"ingest failed:\nevent: {event}\nerror: {e}")
            failure_count += 1
            continue

        records.append(record)
    gdf = gpd.GeoDataFrame(records)

    gdf.set_crs(epsg=4326, inplace=True)  # WGS 84

    gdf = gdf.sort_values(by="date_obj", ascending=False)

    logger.info("{:,} event(s) -ingested-> gdf.".format(len(gdf)))
    metadata["ingested_count"] = len(gdf)
    if failure_count:
        logger.error(f"{failure_count:,} event(s) failed to ingest.")
    metadata["failure_count"] = failure_count

    histogram = Counter(list(gdf["date_obj"].values))

    dates = sorted(histogram.keys())
    logger.info(
        "{:,} day(s) of events, starting {}, until {}.".format(
            len(dates),
            min(dates),
            max(dates),
        )
    )
    metadata["range"] = [min(dates), max(dates)]

    if do_visualize:
        values = [histogram[date] for date in dates]

        plt.figure(figsize=(10, 5))
        plt.bar(dates, values, color="blue")
        plt.xlabel(
            " | ".join(
                [
                    "Date",
                    object_name,
                    f"{NAME}-{VERSION}.{fullname()}",
                ]
            )
        )
        plt.ylabel("# Events / Day")
        plt.title(DESCRIPTION)

        date_count = 20
        if len(dates) > date_count:
            selected_dates = [
                dates[i] for i in range(0, len(dates), len(dates) // date_count)
            ]
            if dates[-1] not in selected_dates:
                selected_dates.append(dates[-1])
        else:
            selected_dates = dates
        plt.xticks(selected_dates, rotation=45)

        plt.tight_layout()
        plt.grid(True)

        if do_save:
            file.save_fig(
                objects.path_of("ukraine_timemap.png", object_name),
                log=log,
            )

    gdf["date"] = gdf["date_obj"].apply(lambda d: f"{d.year}/{d.month:02}/{d.day:02}")

    gdf = gdf.drop(columns=["date_obj"])

    if do_save and not gdf.empty:
        if not file.save_geojson(
            objects.path_of("ukraine_timemap.geojson", object_name),
            gdf,
            log=log,
        ) or not file.save_yaml(
            objects.path_of("metadata.yaml", object_name),
            metadata,
            log=log,
        ):
            return False, gdf

    return True, gdf



from blue_geo.catalog.classes import (
    list_of_catalog_classes,
    list_of_datacube_classes,
    get_catalog,
    get_catalog_class,
    get_collections,
    get_datacube,
    get_datacube_class,
    list_of_catalogs,
)



from typing import List, Type, Union
from blue_geo.catalog.firms import FirmsCatalog
from blue_geo.catalog.generic import (
    GenericCatalog,
    VoidCatalog,
    GenericDatacube,
    VoidDatacube,
)
from blue_geo.catalog.firms.area import FirmsAreaDatacube

list_of_catalog_classes: List[Type[GenericCatalog]] = [
    GenericCatalog,
    FirmsCatalog,
]

list_of_catalogs: List[str] = sorted(
    [catalog_class.name for catalog_class in list_of_catalog_classes]
)

list_of_datacube_classes: List[Type[GenericDatacube]] = [
    GenericDatacube,
    FirmsAreaDatacube,
]


def get_catalog_class(catalog_name: str) -> Type[GenericCatalog]:
    for catalog_class in list_of_catalog_classes:
        if catalog_class.name == catalog_name:
            return catalog_class

    return VoidCatalog


def get_catalog(catalog_name: str) -> GenericCatalog:
    return get_catalog_class(catalog_name)()


def get_collections(
    catalog_class: Union[Type[GenericCatalog], str],
) -> List[Type[GenericDatacube]]:
    if isinstance(catalog_class, str):
        catalog_class = get_catalog_class(catalog_class)

    return [
        datacube_class
        for datacube_class in list_of_datacube_classes
        if datacube_class.catalog.__class__ == catalog_class
    ]


def get_datacube(datacube_id: str) -> GenericDatacube:
    return get_datacube_class(datacube_id)(datacube_id)


def get_datacube_class(datacube_id: str) -> Type[GenericDatacube]:
    for datacube_class in list_of_datacube_classes:
        success, _ = datacube_class.parse_datacube_id(datacube_id)
        if success:
            return datacube_class

    return VoidDatacube



import argparse
from blueness import module
from blue_geo import NAME, VERSION
from abcli.plugins.metadata import get_from_object
from blue_geo.logger import logger
from blueness.argparse.generic import sys_exit

NAME = module.name(__file__, NAME)


parser = argparse.ArgumentParser(NAME, description=f"{NAME}-{VERSION}")
parser.add_argument(
    "task",
    type=str,
    help="read",
)
parser.add_argument(
    "--count",
    type=int,
    default=1,
)
parser.add_argument(
    "--delim",
    type=str,
    default=",",
)
parser.add_argument(
    "--offset",
    type=int,
    default=0,
)
parser.add_argument(
    "--prefix",
    type=str,
    default="",
)
parser.add_argument(
    "--suffix",
    type=str,
    default="",
)
parser.add_argument(
    "--contains",
    type=str,
    default="",
)
parser.add_argument(
    "--notcontains",
    type=str,
    default="",
)
parser.add_argument(
    "--object_name",
    type=str,
)
parser.add_argument(
    "--log",
    default=0,
    type=int,
    help="0|1",
)
parser.add_argument(
    "--show_len",
    default=0,
    type=int,
    help="0|1",
)
args = parser.parse_args()

delim = " " if args.delim == "space" else args.delim

success = False
if args.task == "read":
    success = True
    output = get_from_object(
        args.object_name,
        "datacube_id",
        [],
    )

    output = [
        datacube_id
        for datacube_id in output
        if (not args.prefix or datacube_id.startswith(args.prefix))
        and (not args.suffix or datacube_id.endswith(args.suffix))
        and (not args.contains or args.contains in datacube_id)
        and (not args.notcontains or args.notcontains not in datacube_id)
    ]

    output = output[args.offset :]

    if args.count != -1:
        output = output[: args.count]

    if args.log:
        logger.info(
            "{:,} datacube-id(s){}".format(
                len(output),
                "" if args.show_len else f": {delim.join(output)}",
            )
        )
    else:
        print(len(output) if args.show_len else delim.join(output))
else:
    success = None

sys_exit(logger, NAME, args.task, success)



from typing import Dict
from blue_geo.catalog.firms.area.enums import Area, Source
from datetime import datetime, timedelta

args: Dict[str, Dict] = {
    "area": {
        "default": Area.default().name,
        "help": "|".join(Area.values()),
    },
    "date": {
        "default": (datetime.now() - timedelta(days=5)).strftime("%Y-%m-%d"),
        "help": "yyyy-mm-dd",
    },
    "depth": {
        "type": int,
        "default": 1,
        "help": "1..10",
    },
    "source": {
        "default": Source.default().name,
        "help": "|".join(Source.values()),
    },
}
